#ifndef BUTTONS_H_DEFINED
#define BUTTONS_H_DEFINED

#include "H8.h"
#include "IO_Ports.h"
#include "LCD.h"
#include "Interrupts.h"
#include "Registers.h"

#if defined(DCC_H_DEFINED)
extern void DCC_Init(void);
extern void DCC_Close(void);
#endif

void Run_Handler(void);
void OnOff_Handler(void);

void Run_Handler(void)
{
    Disable;
    Running = !Running;
    if (Running)
    {
        lcd_clear();
        lcd_show_icon(man_run);

#if defined(DCC_H_DEFINED)
        DCC_Init();
#endif
    }
    else
    {
        cputs("Stby  ");
        lcd_show_icon(man_stand);

#if defined(DCC_H_DEFINED)
        DCC_Close();
#endif

    }
    Enable;
}

void OnOff_Handler(void)
{
    Disable;
    RCX_Reset();
    Enable;
}

void Buttons_Init(void)
{
    Disable;

    Running = FALSE;

    IRQ0_Interrupt_Handler = Run_Handler;
    ISCR = bit0;
    IER = bit0;

    IRQ1_Interrupt_Handler = OnOff_Handler;
    ISCR |= bit1;
    IER |= bit1;

    cputs("Stby  ");
    lcd_show_icon(man_stand);

    Enable;
}

void Buttons_Close(void)
{
    Disable;

    ISCR = 0;
    IER = 0;

    Enable;
}

#endif
