#ifndef DCC_H_DEFINED
#define DCC_H_DEFINED

#include "H8.h"
#include "IO_Ports.h"
#include "Registers.h"
#include "Interrupts.h"
#include "Math.h"

//=====================================
// Global Variables

//
static volatile unsigned int DCC_timerCount = 0;
// position in the packet we are at
static volatile unsigned int DCC_packetPosition = 0;
// end of where the packet should be for standard or extended. 47 or 56
static volatile unsigned int DCC_endPacketPosition = 47;
//used in the queue system to repeat packets.
static volatile unsigned int DCC_idlePacketSpace = 0;
//position in the trainArray
static volatile unsigned int DCC_ArrayPosition = 0;
//used for controlling which bank of 3 trains we are on. 1-3, 4-6, 7-9
static int DCC_bank = 0;

//const int DCC_OCRA1 = 30; //61.5us
const int DCC_OCRA1 = 28;
const int DCC_OCRA0 = 80;

typedef enum
{
    standard,
    extended
} DCC_PacketType;


//TODO: need to add a enabled field for queue system.
typedef struct
{
    byte locked; // locked packet for editing
    DCC_PacketType packetType;
    int slot; // train slot
    int address; // save address here. easier to convert from decimal to binary.
    int speed; // save speed here. easier to convert from decimal to binary.
    int packet[56];
    int packetRepeat; //future use: use as part of queue system
} DCC_Packet;


// preamble + '0' + address + '0' + instructions + '0' + errorChecking + '1' (+1)
// extra 1 at the end if due to updating the current packet takes a long time and "streches" the last one, so we'll just add an extra bit to the packet
// so the extra bit will get streched instead.
const int DCC_MaxTrains = 10;
static volatile DCC_Packet DCC_trainArray[10] =
{
    { FALSE, standard, 0, 0xff, 0, { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, 0, 1,1,1,1,1,1,1,1, 0, 0,0,0,0,0,0,0,0, 0, 1,1,1,1,1,1,1,1, 1, 1, 0,0,0,0,0,0,0,0, 0}, 25 }, //idle
    { FALSE, extended, 1, 1, 0,    { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, 0, 0,0,0,0,0,0,0,1, 0, 0,0,1,1,1,1,1,1, 0, 1,0,0,0,0,0,0,0, 0, 1,0,1,1,1,1,1,0, 1, 1}, 3 },  //train 1
    { FALSE, extended, 2, 2, 0,    { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, 0, 0,0,0,0,0,0,1,0, 0, 0,0,1,1,1,1,1,1, 0, 1,0,0,0,0,0,0,0, 0, 1,0,1,1,1,1,0,1, 1, 1}, 3 },  //train 2
    { FALSE, extended, 3, 3, 0,    { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, 0, 0,0,0,0,0,0,1,1, 0, 0,0,1,1,1,1,1,1, 0, 1,0,0,0,0,0,0,0, 0, 1,0,1,1,1,1,0,0, 1, 1}, 3 },  //train 3
    { FALSE, extended, 4, 4, 0,    { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, 0, 0,0,0,0,0,1,0,0, 0, 0,0,1,1,1,1,1,1, 0, 1,0,0,0,0,0,0,0, 0, 1,0,1,1,1,0,1,1, 1, 1}, 3 },  //train 4
    { FALSE, extended, 5, 5, 0,    { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, 0, 0,0,0,0,0,1,0,1, 0, 0,0,1,1,1,1,1,1, 0, 1,0,0,0,0,0,0,0, 0, 1,0,1,1,1,0,1,0, 1, 1}, 3 },  //train 5
    { FALSE, extended, 6, 6, 0,    { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, 0, 0,0,0,0,0,1,1,0, 0, 0,0,1,1,1,1,1,1, 0, 1,0,0,0,0,0,0,0, 0, 1,0,1,1,1,0,0,1, 1, 1}, 3 },  //train 6
    { FALSE, extended, 7, 7, 0,    { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, 0, 0,0,0,0,0,1,1,1, 0, 0,0,1,1,1,1,1,1, 0, 1,0,0,0,0,0,0,0, 0, 1,0,1,1,1,0,0,0, 1, 1}, 3 },  //train 7
    { FALSE, extended, 8, 8, 0,    { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, 0, 0,0,0,0,1,0,0,0, 0, 0,0,1,1,1,1,1,1, 0, 1,0,0,0,0,0,0,0, 0, 1,0,1,1,0,1,1,1, 1, 1}, 3 },  //train 8
    { FALSE, extended, 9, 9, 0,    { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, 0, 0,0,0,0,1,0,0,1, 0, 0,0,1,1,1,1,1,1, 0, 1,0,0,0,0,0,0,0, 0, 1,0,1,1,0,1,1,0, 1, 1}, 3 },  //train 9
};

//=====================================
// FUNCTION PROTOTYPES

void DCC_UpdateAddress(DCC_Packet *, int);
void DCC_Init(void);
void DCC_Close(void);
void DCC_Handler(void);
void DCC_IncreaseSpeed(DCC_Packet *);
void DCC_DecreaseSpeed(DCC_Packet *);
void DCC_Stop(DCC_Packet *);
void DCC_UpdateXOR(DCC_Packet *);


//updates the error checking section of a packet when the speed or address has been modified.
//TODO: modify for extended packets
//address: 19-26
//instruction1: 28-35
//instruction2: 37-44
//xor: 46-53
void DCC_UpdateXOR(DCC_Packet *fullPacket)
{
    int c;

    if (fullPacket->packetType == extended)
    {
        for (c=0; c<8; c++)
        {
            fullPacket->packet[46+c] = ((fullPacket->packet[19+c] ^ fullPacket->packet[28+c]) ^ fullPacket->packet[37+c]);
        }
    }
}


// sets up the FRT timer on the RCX, and sets initials variables.
void DCC_Init(void)
{
    DisableFRT;

    FRT_Interrupt_OCIA_Handler = &DCC_Handler;

    FRT_TCSR = 0;
    FRT_TCSR |= TCSR_RESET_ON_A; //reset counter on output compare a match
    FRT_TCSR &= ~TCSR_OCFA; //clear OCFA
    FRT_TCR = TCR_CLOCK_32; //set prescaler
    FRT_TOCR &= ~TOCR_OCRS; //select OCRA
    FRT_OCRAB_H = 0;
    FRT_OCRAB_L = DCC_OCRA1;

    DCC_timerCount = 0;
    DCC_packetPosition = 0;
    DCC_idlePacketSpace = DCC_trainArray[0].packetRepeat;
    DCC_ArrayPosition = 0;
    DCC_endPacketPosition = 47;

    EnableFRT_OCA;
}

void DCC_Handler(void)
{
    SaveRegisters;
    //reset timer;
    //FRT_TCSR &= ~TCSR_OCFA;

    if (DCC_timerCount == 0)
    {
        //PortA(10);
        OutputPorts = 0x8a;
        DCC_timerCount++;
    }
    else
    {
        //PortA(01);
        OutputPorts = 0x45;
        DCC_timerCount=0;
        DCC_packetPosition++;
    }

    //DisableFRT;
    if (DCC_trainArray[DCC_ArrayPosition].packet[DCC_packetPosition] == 1)
    {
        FRT_OCRAB_H = 0;
        FRT_OCRAB_L = DCC_OCRA1;
    }
    else
    {
        FRT_OCRAB_H = 0;
        FRT_OCRAB_L = DCC_OCRA0;
    }

    //EnableFRT_OCA;

    if (DCC_packetPosition >= DCC_endPacketPosition)
    {
        DCC_packetPosition = 0;
        DCC_idlePacketSpace--;

        if (DCC_idlePacketSpace <= 0)
        {
            //unlock current train packet
            DCC_trainArray[DCC_ArrayPosition].locked == FALSE;
            //find next train in the array
            do
            {
                DCC_ArrayPosition++;
            }
            while ((DCC_trainArray[DCC_ArrayPosition].locked == TRUE) && (DCC_ArrayPosition < (DCC_MaxTrains-1) ));

            if (DCC_ArrayPosition > (DCC_MaxTrains-1))
            {
                DCC_ArrayPosition=0;
            }

            //lock current train packet
            if (DCC_ArrayPosition != 0)
            {
                DCC_trainArray[DCC_ArrayPosition].locked == TRUE;
            }

            DCC_idlePacketSpace = DCC_trainArray[DCC_ArrayPosition].packetRepeat;

            if (DCC_trainArray[DCC_ArrayPosition].packetType == standard)
            {
                DCC_endPacketPosition = 47;
            }
            else
            {
                DCC_endPacketPosition = 56;
            }

        }
    }

    FRT_TCSR &= ~TCSR_OCFA;

    RestoreRegisters;
}

//used to disable the timer and set output ports to float
void DCC_Close(void)
{
    DisableFRT;
    OutputPorts = 0;
}

//future use
// provides the ability to set the address on a packet instead of hard coding 1-9.
//TODO: modify for extended packets and 2byte addressing
void DCC_UpdateAddress(DCC_Packet *fullPacket, int address)
{
    int t[8];
    int c;

    if (fullPacket->locked == FALSE)
    {
        fullPacket->locked = TRUE;

        fullPacket->address = address;
        DecimalToBinary(t, fullPacket->address);

        for(c=18; c<26; c++)
        {
            fullPacket->packet[c] = t[c-18];
        }
        DCC_UpdateXOR((DCC_Packet *)fullPacket);

        fullPacket->locked = FALSE;
    }
}

//add 1 to the speed counter in a packet, convert to binary and write into the packet
//37-44
void DCC_IncreaseSpeed(DCC_Packet *fullPacket)
{
    int t[8];
    int c;

    if (fullPacket->locked==FALSE)
    {
        fullPacket->locked = TRUE;

        if (fullPacket->speed < 126)
        {
            (fullPacket->speed)++;
            DecimalToBinary(t, fullPacket->speed);

            if (fullPacket->speed > 0)
            {
                fullPacket->packet[37] = 1;

            }
            for(c=38; c<44; c++)
            {
                fullPacket->packet[c] = t[c-37];
            }

            DCC_UpdateXOR((DCC_Packet *)fullPacket);
        }
        fullPacket->locked = FALSE;
    }

    lcd_int(fullPacket->speed);
    lcd_digit(fullPacket->slot);
    lcd_refresh();
}

//subtract 1 from the speed counter in a packet, convert to binary and write into the packet
//37-44
void DCC_DecreaseSpeed(DCC_Packet *fullPacket)
{
    int t[8];
    int c;

    if (fullPacket->locked==FALSE)
    {
        fullPacket->locked = TRUE;

        if (fullPacket->speed > -126)
        {
            (fullPacket->speed)--;
            DecimalToBinary(t, (int)fullPacket->speed);

            //set direction flag
            if (fullPacket->speed < 0)
            {
                fullPacket->packet[37] = 0;
            }
            for(c=38; c<44; c++)
            {
                fullPacket->packet[c] = t[c-37];
            }

            DCC_UpdateXOR((DCC_Packet *)fullPacket);
        }
        fullPacket->locked = FALSE;
    }

    lcd_digit(fullPacket->slot);
    lcd_int(fullPacket->speed);
    lcd_refresh();
}

//set speed 0, convert to binary and write into the packet
//37-44
void DCC_Stop(DCC_Packet *fullPacket)
{
    int c;

    if (fullPacket->locked==FALSE)
    {
        fullPacket->locked = TRUE;

        fullPacket->speed = 0;
        for(c=38; c<44; c++)
        {
            fullPacket->packet[c] = 0;
        }
        DCC_UpdateXOR((DCC_Packet *)fullPacket);

        fullPacket->locked = FALSE;
    }

    lcd_digit(fullPacket->slot);
    lcd_int(fullPacket->speed);
    lcd_refresh();
}


#endif
