#ifndef IR_H_DEFINED
#define IR_H_DEFINED

#include "H8.h"
#include "Interrupts.h"
#include "IO_Ports.h"
#include "Registers.h"
#include "Remote_Control.h"

#define DisableIR		SCI_SCR &= ~(bit7 | bit6 | bit2)
#define EnableIR_Receive	SCI_SCR |= bit6
#define EnableIR_Transmit	SCI_SCR |= bit7

#define LongRange	P4DR &= ~bit0
#define ShortRange	P4DR |= bit0

#define Header	0x55
#define FF	0xff
#define Zero	0x00

static volatile word irState = 0;

typedef enum
{
    LookingForHeader,
    HeaderReceived,
    FFReceived,
    ZeroReceived,
    b7Received,
    b6Received,
    b5Received,
    b4Received,
    b3Received,
    b2Received,
    b1Received,
    b0Received
} Rstate;

static volatile Rstate ReceiveState;

struct IR_buffer
{
    byte empty;
    byte b7, b6, b5, b4, b3, b2, b1, b0;
};
static volatile struct IR_buffer ReceiveBuffer;

static byte IR_Error;

void RXI_Handler(void)
{
    byte b;
    SaveRegisters;

    SCI_SSR &= ~bit6;

    if (ReceiveBuffer.empty)
    {

        b = SCI_RDR;

        switch (ReceiveState)
        {
        case LookingForHeader:
            if (b == Header)
                ReceiveState = HeaderReceived;
            break;
        case HeaderReceived:
            if (b == FF)
                ReceiveState = FFReceived;
            else
                ReceiveState = LookingForHeader;
            break;
        case FFReceived:
            if (b == Zero)
                ReceiveState = ZeroReceived;
            else
                ReceiveState = LookingForHeader;
            break;
        case ZeroReceived:
            ReceiveBuffer.b7 = b;
            ReceiveState = b7Received;
            break;
        case b7Received:
            ReceiveBuffer.b6 = b;
            ReceiveState = b6Received;
            if ((ReceiveBuffer.b6 ^ ReceiveBuffer.b7) != 0xFF)
            {
                ReceiveState = LookingForHeader;
            }
            break;
        case b6Received:
            ReceiveBuffer.b5 = b;
            ReceiveState = b5Received;
            break;
        case b5Received:
            ReceiveBuffer.b4 = b;
            ReceiveState = b4Received;
            if ((ReceiveBuffer.b5 ^ ReceiveBuffer.b4) != 0xFF)
            {
                ReceiveState = LookingForHeader;
            }
            break;
        case b4Received:
            ReceiveBuffer.b3 = b;
            ReceiveState = b3Received;
            break;
        case b3Received:
            ReceiveBuffer.b2 = b;
            ReceiveState = b2Received;
            if ((ReceiveBuffer.b3 ^ ReceiveBuffer.b2) != 0xFF)
            {
                ReceiveState = LookingForHeader;
            }
            break;
        case b2Received:
            ReceiveBuffer.b1 = b;
            ReceiveState = b1Received;
            break;
        case b1Received:
            ReceiveBuffer.b0 = b;
            ReceiveState = b0Received;
            break;
        case b0Received:
            if ((ReceiveBuffer.b6 ^ ReceiveBuffer.b7) != 0xFF)
            {
                ReceiveBuffer.empty = TRUE;
            }
            else
            {
                ReceiveBuffer.empty = FALSE;
            }
            ReceiveState = LookingForHeader;
            break;
        }
    }

    RestoreRegisters;
}

void ERI_Handler(void)
{
    Disable;
    SaveRegisters;

    IR_Error = (SCI_SSR & (bit5 | bit4 |bit3 )) >> 3;

    SCI_SSR &= ~(bit6 | bit5 | bit4 | bit3);

    ReceiveState = LookingForHeader;

    SCI_SCR &= ~bit7;
    SCI_SCR &= ~bit2;

    RestoreRegisters;
    Enable;
}

byte IR_Receive()
{
    byte result = FALSE;

    if (ReceiveBuffer.empty == FALSE)
    {
        result = TRUE;
        irState = (ReceiveBuffer.b5 << 8) + ReceiveBuffer.b3;
        ReceiveBuffer.empty = TRUE;
    }

    return result;
}

void IR_Init(void)
{
    ReceiveState = LookingForHeader;
    ReceiveBuffer.empty = TRUE;
    ReceiveBuffer.b0 = 0;
    ReceiveBuffer.b1 = 0;
    ReceiveBuffer.b2 = 0;
    ReceiveBuffer.b3 = 0;
    ReceiveBuffer.b4 = 0;
    ReceiveBuffer.b5 = 0;
    ReceiveBuffer.b6 = 0;
    ReceiveBuffer.b7 = 0;
    IR_Error = 0;

    P4DDR_ROM |= bit0;
    P4DDR = P4DDR_ROM;
    ShortRange;

    SCI_SCR = 0;
    SCI_SMR |= (bit5 | bit4);
    SCI_BRR = 207;
    SCI_SCR |= (bit5 |bit4);

    RXI_Interrupt_Handler = &RXI_Handler;
    ERI_Interrupt_Handler = &ERI_Handler;

    SCI_SCR |= (bit5 | bit4);

    EnableIR_Receive;
}

void IR_Close(void)
{
    SCI_SCR = 0;
}

#endif
