#include "LCD.h"
#include "String.h"
#include "Time.h"

#ifndef   LCD_ASCII_H_DEFINED
#define   LCD_ASCII_H_DEFINED

const char ascii_display_codes[] =
{
  0x00, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,	// non-printables
   0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,	// -> underscore
   0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,	// except 0x00.
   0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,

	0x00,			// 32 ' '
	0x42,			// 33 ! 1
	0x0a,			// 34 "
	0x7b,			// 35 #
	0x6d,			// 36 $ 5 S Z
	0x13,			// 37 % /
	0x7d,			// 38 & 6
	0x08,			// 39 '  alt: `
	0x3c,			// 40 ( C [ {
	0x66,			// 41 ) ] }
	0x5b,			// 42 * H K X
	0x43,			// 43 +  alt: 0x19 (worse)
	0x10,			// 44 , .  alt: 0x40
	0x01,			// 45 -
	0x10,			// 46 . alt: 0x40
	0x13,			// 47 /
	0x7e,			// 48 0 0
	0x42,			// 49 1
	0x37,			// 50 2
	0x67,			// 51 3
	0x4b,			// 52 4
	0x6d,			// 53 5
	0x7d,			// 54 6
	0x46,			// 55 7
	0x7f,			// 56 8
	0x6f,			// 57 9
	0x21,			// 58 : ; = alt:0x5 (worse)
	0x21,			// 59 ;
	0x31,			// 60 < c alt:0xd (worse)
	0x21,			// 61 =
	0x61,			// 62 >   alt: 0x7 (worse)
	0x17,			// 63 ?
	0x3f,			// 64 @ alt: e
	0x5f,			// 65 A
	0x79,			// 66 b
	0x3c,			// 67 C
	0x73,			// 68 d
	0x3d,			// 69 E
	0x1d,			// 70 F
	0x7c,			// 71 G
	0x5b,			// 72 H
	0x42,			// 73 I 1
	0x62,			// 74 J
	0x5b,			// 75 K
	0x38,			// 76 L
	0x5e,			// 77 M N
	0x5e,			// 78 N
	0x7e,			// 79 O alt: o
	0x1f,			// 80 P
	0x4f,			// 81 Q
	0x11,			// 82 r
	0x6d,			// 83 S
	0x46,			// 84 T alt: t
	0x7a,			// 85 U V W
	0x7a,			// 86 V
	0x7a,			// 87 W
	0x5b,			// 88 X
	0x6b,			// 89 Y
	0x37,			// 90 Z
	0x3c,			// 91 [
	0x49,			// 92 '\\'
	0x66,			// 93 ]
	0x0e,			// 94 ^ ~
	0x20,			// 95 _
	0x02,			// 96 ` alt: '
	0x5f,			// 97 A R
	0x79,			// 98 b
	0x31,			// 99 c
	0x73,			// 100 d
	0x3d,			// 101 E
	0x1d,			// 102 F
	0x7c,			// 103 G
	0x59,			// 104 h
	0x42,			// 105 I 1
	0x62,			// 106 J alt 0x60 (worse)
	0x5b,			// 107 K alt h (worse?)
	0x38,			// 108 L
	0x51,			// 109 m n
	0x51,			// 110 n
	0x71,			// 111 o
	0x1f,			// 112 P
	0x4f,			// 113 q
	0x11,			// 114 r
	0x6d,			// 115 S
	0x39,			// 116 t
	0x70,			// 117 u v w
	0x70,			// 118 v
	0x70,			// 119 w
	0x5b,			// 120 X
	0x6b,			// 121 Y
	0x37,			// 122 Z
	0x3c,			// 123 {
	0x18,			// 124 | (left) alt: 1 (worse)
	0x66,			// 125 }
	0x0e,			// 126 ~
	0x00			// 127 "" 127 empty
};

//! display an ASCIIZ string
/*! \param s the string

   only the first 5 characters will be displayed.
   if there are less than 5 characters, the remaining display
   positions will be cleared.
 */

void cputc(char c, int pos)
{
	cputc_native(ascii_display_codes[(c) & 0x7f], pos);
	lcd_refresh();
}

void cputs(const char *s)
{
  int i;

  cputc_native(0, 5);
  for (i = 4; (*s) && (i >= 0);)
    cputc(*(s++), i--);
  while (i >= 1)
    cputc_native(0, i--);

  lcd_refresh();
}

//! clear user portion of screen
void cls() {
  cputs("    ");
}

void lcd_marquee(char *s) {
    int c;

    //loop through string
	for (c=0;c<length(s);c++) {
        if (c >= 0) {
            cputc(s[c], 0);
        }
        if (c >= 1) {
            cputc(s[c-1], 1);
        }
        if (c >= 2) {
            cputc(s[c-2], 2);
        }
        if (c >= 3) {
            cputc(s[c-3], 3);
        }
        if (c >= 4) {
            cputc(s[c-4], 4);
        }
        BusyWait_MS(25000);
	}

}

#endif /* LCD_ASCII_H_DEFINED */
