#include "H8.h"
#include "IO_Ports.h"

#ifndef REGISTERS_H_DEFINED
#define REGISTERS_H_DEFINED

//serial/time control register
#define STCR	*((volatile byte *) 0xffc3)
//MCU Operating Mode and Address Space
#define SYSCR	*((volatile byte *) 0xffc4)
// IRQ Sense Control Register
#define ISCR	*((volatile byte *) 0xffc6)
//IRQ Enable Register
#define IER	*((volatile byte *) 0xffc7)

//FRT Registers
#define FRT_TIER        *((volatile byte *) 0xff90) //timer interrupt enable register
#define FRT_TCSR        *((volatile byte *) 0xff91) //Timer control/status register
#define FRT_FRC         *((volatile byte *) 0xff92) //Free-running Counter
#define FRT_OCRAB_H        *((volatile byte *) 0xff94) //Output compare register A/B
#define FRT_OCRAB_L        *((volatile byte *) 0xff95) //Output compare register A/B
#define FRT_TCR         *((volatile byte *) 0xff96) //timer control register
#define FRT_TOCR        *((volatile byte *) 0xff97) //timer output compare control register
#define FRT_ICRA        *((volatile byte *) 0xff98) //input capture register a
#define FRT_ICRB        *((volatile byte *) 0xff9a) //input capture register b
#define FRT_ICRC        *((volatile byte *) 0xff9c) //input capture register c
#define FRT_ICRD        *((volatile byte *) 0xff9e) //input capture register d


#define DisableFRT	FRT_TIER = 0
#define EnableFRT_OCA	FRT_TIER |= TIER_ENABLE_OCIA
#define DisableFRT_OCA	FRT_TIER &= ~TIER_ENABLE_OCIA
#define EnableFRT_OCB	FRT_TIER |= TIER_ENABLE_OCIB
#define DisableFRT_OCB	FRT_TIER &= ~TIER_ENABLE_OCIB

//TIER bitmasks
#define TIER_ENABLE_OCIA		0x08    //Output Compare Interrupt A
#define TIER_ENABLE_OCIB		0x04    //Output Compare Interrupt B
#define TIER_ENABLE_OVIE        0x02    //Overflow Interrupt Enable

//TCSR bitmasks
#define TCSR_ICFA           0x80        //Input Capture Flag A
#define TCSR_ICFB           0x40        //Input Capture Flag B
#define TCSR_ICFC           0x20        //Input Capture Flag C
#define TCSR_ICFD           0x10        //Input Capture Flag D
#define TCSR_OCFA           0x08        //Output Compare Flag A
#define TCSR_OCFB           0x04        //Output Compare Flag B
#define TSCR_OVF            0x02        //Overflow Flag
#define TCSR_RESET_ON_A		0x01        //Counter Clear A

//TCR bitmasks
#define TCR_IEDGA           0x80        //Input Edge Select A
#define TCR_IEDGB           0x40        //Input Edge Select B
#define TCR_IEDGC           0x20        //Input Edge Select C
#define TCR_IEDGD           0x10        //Input Edge Select D

//16Mhz / clock prescaler (16/32)=500    clock cyle is equal to 1000000/500000=2us
#define TCR_CLOCK_32		0x02	   // clock = pclock / 32   500KHZ  2us
#define TCR_CLOCK_8			0x01	   // clock = pclock / 8    4MHZ    250ns
#define TCR_CLOCK_2			0x00	   // clock = pclock / 2    8MHZ    125ns



//TOCR bitmasks
#define TOCR_OCRS           0x10
#define TOCR_OEA    		0x08
#define TOCR_OEB            0x04
#define TOCR_OLVLA    		0x02
#define TOCR_OLVLB          0x01

// Serial Communication Interface
#define SCI_SMR	*((volatile byte *) 0xffd8)
#define SCI_BRR	*((volatile byte *) 0xffd9)
#define SCI_SCR *((volatile byte *) 0xffda)
#define SCI_TDR *((volatile byte *) 0xffdb)
#define SCI_SSR *((volatile byte *) 0xffdc)
#define SCI_RDR *((volatile byte *) 0xffdd)

#endif

