;/*
 *  rcx.lds
 *
 *  GNU ld script for mapping a C compiled object module into
 *  an S-record represented absolute H8/300 program. The sections of
 *  the object module is mapped into the memory of the RCX starting at
 *  the load address 0x8000. The length of the program should not exceed
 *  0x6fff because the address 0xf000 is used as device register.
 *  The program entry point is __start.
 *coff-h8300
*/

OUTPUT_FORMAT(srec)
OUTPUT_ARCH(h8300)
ENTRY(_start)
MEMORY
{
    rom (r)  : o = 0x0000, l = 0x8000
    mem : ORIGIN = 0x8000, LENGTH = 0x6fff
}
SECTIONS
{

  .rom : {
    /* used rom functions */

    _rom_reset_vector = 0x0000;

    _reset        = 0x03ae ;
    rom_memcpy    = 0x042a ;
    lcd_show      = 0x1b62 ;
    lcd_hide      = 0x1e4a ;
    lcd_number    = 0x1ff2 ;
    lcd_clear     = 0x27ac ;
    lcd_refresh   = 0x27c8 ;
    power_init    = 0x2964 ;
    sound_system  = 0x299a ;
    power_off     = 0x2a62 ;
    sound_playing = 0x3ccc ;

  } > rom

    .text : {
        *(.init)
        *(.text)
        *(.rodata)
    } > mem
    .data : {
        *(.data)
    } > mem
    .bss : {
        __bss_start = . ;
        *(.bss)
        *(COMMON)
        __end = . ;
    } > mem

    /DISCARD/ : {
        *(.vectors)
    }
}
